% Comparison of openings with different line SEs
% Methods 1,2,3,5: once, angle=0.4, length=40.
% Method 4: all angles, lenght=40 ; all lengths, angle=0.4.
% Method 6: all angles, length=[1,40] ; all lengths, angle = 0.4.

function line_se_comparison

m = {'discrete_line','bresenham_line','soille','interpolated_line',...
     'true_interpolated','grey-value'}; Nm = length(m);
t_phi = [0.1,0.2,0.3,0.4,0.5,0.6,0.7]; Nphi = length(t_phi);
t_l = [5,10,20,40]; Nl = length(t_l);
err = cell(Nphi,Nl,Nm);
time = zeros(Nphi,Nl,Nm);
phi_samples = [-0.15:0.01:0.15];
l_samples = [-4:0.25:4];
params = [4,4,4,4,4,4   % ii=angle
          4,4,4,4,4,4   % jj=length
          1,2,3,4,5,6]; % kk=method
params = sortrows(params')';
ii = 0; jj = 0; % make sure an image is generated the first time around.
for r = 1:size(params,2)
   if ii~=params(1,r) | jj~=params(2,r)
      % If the parameters are the same, don't generate the image again.
      ii = params(1,r);
      jj = params(2,r);
      if exist(['line_se_test_image_',num2str(t_l(jj)),'_',num2str(t_phi(ii)),'.ics'])
         disp(['***reading in image: l=',num2str(t_l(jj)),', phi=',num2str(t_phi(ii))])
         in = readim(['line_se_test_image_',num2str(t_l(jj)),'_',num2str(t_phi(ii))])
      else
         disp(['***generating image: l=',num2str(t_l(jj)),', phi=',num2str(t_phi(ii))])
         in = create_image(t_phi(ii),t_l(jj));
         mask = label(bdilation(in>0.1,1,2));
         phi = t_phi(ii) + phi_samples;
         l = t_l(jj) + l_samples;
         normfactor = double(measure(mask,in,'sum'));
         normfactor = reshape(normfactor,[1,1,length(normfactor)]);
         normfactor = repmat(normfactor,[length(phi),length(l),1]);
         writeim(in,['line_se_test_image_',num2str(t_l(jj)),'_',num2str(t_phi(ii))],'ICSv2');
      end
   end
   kk = params(3,r);
   disp([m{kk},' l=',num2str(t_l(jj)),', phi=',num2str(t_phi(ii))]);
   tic;
   err{ii,jj,kk} = test_method(m{kk},in,mask,phi,l)./normfactor;
   time(ii,jj,kk) = toc;
   disp(['   ',num2str(time(ii,jj,kk)),'s'])
end
clear in mask ii jj kk l phi r normfactor Nm Nphi Nl
save line_se_comparison

for r = 1:size(params,2)
   ii = params(1,r);
   jj = params(2,r);
   kk = params(3,r);
   if 1
      complete_results(err{ii,jj,kk},t_phi(ii)+phi_samples,t_l(jj)+l_samples,[m{kk},'_',num2str(ii),'_',num2str(jj)])
      %complete_results_std(err{ii,jj,kk},t_phi(ii)+phi_samples,t_l(jj)+l_samples,[m{kk},'_',num2str(ii),'_',num2str(jj)])
   else
      T = err{ii,jj,kk};
      T = -diff(T,1,2);
      disp([m{kk},'_',num2str(ii),'_',num2str(jj)])
      disp(sum(T(:)<0)) % number of violations
      disp(sum(T(T<0))) % seriousness of violations
      dipshow(0,mean(err{ii,jj,kk},3)')
   end
end

if 0
   % Create image containing expected result of test
   phi = 0.4;
   l = 40;
   phi_samples = [-0.15:0.01:0.15]+phi;
   l_samples = [-4:0.25:4]+l;
   im = newim(length(phi_samples),length(l_samples));
   im(find(phi_samples==phi)-1,0:(find(l_samples==l)-1)) = 255;
   figure
   set(gcf,'position',[300,300,300,300])
   colormap(gray(256))
   image(l_samples,phi_samples,(uint8(im))')
   set(gca,'position',[0.05,0.15,0.77,0.8])
   axis tight
   set(gca,'YAxisLocation','right','YDir','normal','XDir','reverse')
   xlabel('Length of SE')
   ylabel('Anlge of SE')
   set(gcf,'PaperPositionMode','auto')
   print('-deps2','-r600','comparison_ideal.eps')
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out = create_image(t_phi,t_l)
out = newim(256,256);
for ii=30:20:225
   for jj=27:50:227 % 50:50:200
      % (From BANDLIMITEDLINE(out,l,phi) in LINE_SE.)
      x = xx(out,'corner')-jj+randn(1)/2;
      y = yy(out,'corner')-ii+randn(1)/2;
      rx = x*cos(t_phi) + y*sin(t_phi);
      rx = (t_l/2)-abs(rx);
      ry = y*cos(t_phi) - x*sin(t_phi);
      rx(abs(ry)>4) = -4; % avoid computations
      ry(rx<-4) = -4;     % avoid computations
      a1 = gaussianedgeclip(rx,1);
      a2 = gaussianlineclip(ry,1,1);
      out = max(out,a1*a2);
   end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function err = test_method(method,in,mask,phi,l)
err = zeros(length(phi),length(l),max(mask));
for ii=1:length(phi)
   for jj=1:length(l)
      out = line_se('opening',in,l(jj),phi(ii),method);
      err(ii,jj,:) = double(measure(mask,out,'sum'));
   end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_results(err,phi,l,tit)
err = mean(err,3)';
clf
set(gcf,'position',[300,300,300,300])
colormap([128:255;128:255;128:255]'/255)
surf(phi,l,err,'facecolor',[0.75,0.75,0.75],'edgecolor',[0,0,0])
axis tight
set(gca,'box','on','projection','perspective','zlim',[0,1],'xtick',unique(round(phi*10))/10)
zlabel('Sum over image','rotation',98)
xlabel('Anlge of SE','rotation',74)
ylabel('Length of SE','rotation',-13)
view(-73.5,62)
h = camlight('left');
lighting phong
material dull
%set(h,'style','infinite','position',[-0.5,0.5,0.08])
set(gcf,'PaperPositionMode','auto')
print('-deps2','-r600',['comparison_',tit,'.eps'])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function image_results(err,phi,l,tit)
err = mean(err,3)';
clf
set(gcf,'position',[300,300,300,300])
colormap(gray(256))
image(phi,l,uint8(err*255))
axis tight
ylabel('Length of SE')
xlabel('Anlge of SE')
set(gcf,'PaperPositionMode','auto')
print('-deps2','-r600',['comparison_',tit,'_flat.eps'])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function complete_results(err,phi,l,tit)
err = mean(err,3)';
clf
set(gcf,'position',[300,300,300,350])
colormap(gray(256))

col = (0.5+(err/2))*255;
h = surf(phi,l,err,col,'facecolor',[0.75,0.75,0.75],'edgecolor',[0,0,0]);
set(gca,'position',[0.14,0.10,0.78,0.7]);
axis tight
set(gca,'box','on','projection','perspective','zlim',[0,1],'xtick',unique(round(phi*10))/10)
zlabel('Sum over image','rotation',98)
xlabel('Anlge of SE','rotation',74)
ylabel('Length of SE','rotation',-13)
view(-73.5,62)
h = camlight('left');
lighting phong
material dull

axes('position',[0.50,0.63,0.385,0.33],'FontSize',7)
image(l,phi,(uint8(err*255))')
axis tight
set(gca,'YAxisLocation','right','YDir','normal','XDir','reverse','YTickMode','manual')
xlabel('Length of SE')
ylabel('Anlge of SE')

set(gcf,'PaperPositionMode','auto')
print('-deps2','-r300',['comparison_',tit,'.eps'])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function complete_results_std(err,phi,l,tit)
err = std(err,[],3)';
clf
set(gcf,'position',[300,300,300,350])
colormap(gray(256))

surf(phi,l,err,(err+max(err(:))/2),'facecolor',[0.75,0.75,0.75],'edgecolor',[0,0,0])
set(gca,'position',[0.14,0.10,0.78,0.7])
axis tight
set(gca,'box','on','projection','perspective','zlim',[0,0.05],'xtick',unique(round(phi*10))/10)
if any(err>0.05)
   disp(['Note scaling of y-axis in ',tit,'.'])
   set(gca,'zlim',[0,max(err(:))]);
end
view(-73.5,62)
%zlabel('Standard deviation','rotation',98)
if any(err>0.05)
   zlabel('Standard deviation','position',[phi(1),l(end),max(err(:))*1.2],'horizontalalignment','left')
else
   zlabel('Standard deviation','position',[phi(1),l(end),0.06],'horizontalalignment','left')
end
xlabel('Anlge of SE','rotation',74)
ylabel('Length of SE','rotation',-13)
h = camlight('left');
lighting phong
material dull
%set(h,'style','infinite','position',[-0.5,0.5,0.08])

axes('position',[0.58,0.63,0.385,0.33],'FontSize',7)
if any(err>0.05)
   image(phi,l,uint8(stretch(err)))
else
   image(phi,l,uint8(err*(255/0.05)))
end
axis tight
ylabel('Length of SE')
xlabel('Anlge of SE')

set(gcf,'PaperPositionMode','auto')
print('-deps2','-r600',['comparison_',tit,'_std.eps'])
