% Timing of the different line SEs
%
% Call
%   line_se_timing
% once to do the computations, then
%   line_se_timing(method)
% for each method (1:6) to plot the graphs.

function line_se_timing(method)

if nargin==0

   t_m = {'discrete_line','bresenham_line','soille','interpolated_line',...
        'true_interpolated','grey-value'}; Nm = length(t_m);
   t_phi = 0:pi/16:pi/4; Nphi = length(t_phi);
   t_l = 10:5:30; Nl = length(t_l);
   t_N = 200:100:600; NN = length(t_N);
   time = zeros(NN,Nphi,Nl,Nm);

   for i_N = 1:NN
      disp(['***generating image: N=',num2str(t_N(i_N))])
      in = create_image(t_N(i_N));
      writeim(in,['line_se_test_image_',num2str(t_N(i_N))],'ICSv2');
      disp('   Computing...')
      for i_phi = 1:Nphi
      for i_l = 1:Nl
      for i_m = 1:Nm
         tic;
         line_se('closing',in,t_l(i_l),t_phi(i_phi),t_m{i_m});
         line_se('closing',in,t_l(i_l),t_phi(i_phi),t_m{i_m});
         line_se('closing',in,t_l(i_l),t_phi(i_phi),t_m{i_m});
         line_se('closing',in,t_l(i_l),t_phi(i_phi),t_m{i_m});
         time(i_N,i_phi,i_l,i_m) = toc/4;
      end
      end
      end
   end
   clear in i_* ans
   save line_se_timing

else

   load line_se_timing
   figure('name','image size vs SE lenght and angle')
   maxval = time(:,:,:,method); maxval = max(maxval(:));
   for ii=1:Nphi, for jj=1:Nl
      subplot(Nphi,Nl,jj+(ii-1)*Nl,'align')
      plot(t_N.^2,squeeze(time(:,ii,jj,method)))
      set(gca,'xlim',[0,max(t_N.^2)],'ylim',[0,maxval])
   end, end
   figure('name','SE angle vs length and image size')
   for ii=1:NN, for jj=1:Nl
      subplot(NN,Nl,jj+(ii-1)*Nl,'align')
      switch method
      case 3
         x = zeros(Nphi,1);
         for kk=1:Nphi
            rect = round(t_l(jj)*[cos(t_phi(kk)),sin(t_phi(kk))]);
            x(kk) = periodicity(rect);
         end
         scatter(x,squeeze(time(ii,:,jj,method)))
         set(gca,'xlim',[0,max(x)],'ylim',[0,maxval])
      case 6
         plot(sin(t_phi).*cos(t_phi),squeeze(time(ii,:,jj,method)))
         set(gca,'xlim',[0,max(sin(t_phi).*cos(t_phi))],'ylim',[0,maxval])
      otherwise
         plot(t_phi,squeeze(time(ii,:,jj,method)))
         set(gca,'xlim',[0,max(t_phi)],'ylim',[0,maxval])
      end
   end, end
   figure('name','SE length vs angle and image size')
   for ii=1:NN, for jj=1:Nphi
      subplot(NN,Nphi,jj+(ii-1)*Nphi,'align')
      plot(t_l,squeeze(time(ii,jj,:,method)))
      set(gca,'xlim',[0,max(t_l)],'ylim',[0,maxval])
   end, end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function out = create_image(N)
out = newim(N,N);
t_phi = pi/8;
t_l = 40;
for ii=30:20:(N-30)
   for jj=27:50:(N-27)
      % (From BANDLIMITEDLINE(out,l,phi) in LINE_SE.)
      x = xx(out,'corner')-jj+randn(1)/2;
      y = yy(out,'corner')-ii+randn(1)/2;
      phi = t_phi+(rand(1)*pi/10)-pi/20;
      rx = x*cos(phi) + y*sin(phi);
      rx = (t_l/2)-abs(rx);
      ry = y*cos(phi) - x*sin(phi);
      rx(abs(ry)>4) = -4; % avoid computations
      ry(rx<-4) = -4;     % avoid computations
      a1 = gaussianedgeclip(rx,1);
      a2 = gaussianlineclip(ry,1,1);
      out = max(out,a1*a2);
   end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function p = periodicity(rect)
if any(mod(rect,1)~=0)
   error('rect must be integer valued')
end
q = max(rect);
r = min(rect);
p = q;
for d = r:-1:2
   if mod(r/d,1)==0 & mod(q/d,1)==0
      p = q/d;
      break;
   end
end
